-------------------------------------------------------
-- Design Name : com_adc_dec 
-- File Name   : com_adc_dec.vhd
-- ADC chip    : AD9649
-- Function    : serial data recovery from comm. ADC output signals 
-- Coder       : K.-H. Sulanke, DESY
-- Date        : 2020-01-11
-- Revision    : 14
-------------------------------------------------------
-- to be used for IceCube gen1 encoding style, decoding the trailing edge only
-- ADC resolution is 2V / 14 bit = 0,122 mV / bit -> ~ 8bit / 1mV 
-- based on decoding of the down-going signal edge
-- 2020-01-04, rev. 13, inspired by old DOR decoder (AHDL)
-- 2020-01-04, rev. 14, adc_sum pipeline increased from 4 to 8, com_adc_thr increased accordingly

library ieee;
    use ieee.std_logic_1164.all;
    use ieee.std_logic_unsigned.all;
    use ieee.numeric_bit.all;
    use ieee.std_logic_arith.all;
Library UNISIM;
    use UNISIM.vcomponents.all;

entity com_adc_dec is
   port(
         reset           : in  std_logic;
         clk             : in  std_logic; -- 60 MHz comm. clock
		 rx_ena          : in  std_logic;
         com_thr_adj     : in  std_logic_vector (1 downto 0); -- set the comm. threshold
         COM_ADC_CSBn    : out std_logic;   --
         COM_ADC_SCLK    : out std_logic;   --
         COM_ADC_SDIO    : inout std_logic; --
         COM_ADC_D       : in  std_logic_vector (13 downto 0); -- available 3ns after COM_ADC_CLK-LH edge
         COM_ADC_CLK_N   : out std_logic;  --
         COM_ADC_CLK_P   : out std_logic;  -- 
         com_adc_dec_sdout : out std_logic;  -- decoder serial data output
         hl_edge_test    : out std_logic  -- for debugging
--       du              : in  integer range 0 to 2**14-1    -- typical transistion / fall time is 13 mV / ADC tick (33ns)        
       );
end entity;

architecture com_adc_dec_arch of com_adc_dec is

  constant ADC_CLK          : integer := 30_000_000;
  constant BAUD_RATE        : integer :=  2_000_000;  -- 3_000_000
  constant MAX_PULSE_LENGTH : integer:= ADC_CLK * 2 / BAUD_RATE; -- each '1' pulse corresponds to ... rx_ena pulses or ADC clocks
 
  constant mV : integer := 8; -- 8 ADC bins per 1mV 
   
  type   array_Nx14 is array (0 to 3) of integer range 0 to 2**14 -1; 
  signal com_adc_pipe          : array_Nx14 := ( others => 0);
  
  type   array_Nx16 is array (0 to 7) of integer range 0 to 2**16 -1; 
  signal com_adc_sum           : array_Nx16 := ( others => 0);  
  
  signal hl_edge               : std_logic; 
 -- signal lh_edge           : std_logic;
  
  signal com_adc_clock         : std_logic := '0';
  signal com_adc_thr           : integer range 0 to 2**16-1;
  signal com_adc_diff          : integer range -2**16 to 2**16-1; -- IMPORTANT !!! all were natural before with 0 to ..

  signal com_adc_dec_sdout_nd  : std_logic;
  signal pulse_length_ct       : integer range 0 to MAX_PULSE_LENGTH;

  
 begin      
 
 -- recommended thresholds, 2Mbd, depending on the encoding:
 -- rectangular    : 100 * mV, min = 30 * mV (?), all use cases,
 --                : 800 * mV, short cable
 -- ramped (5-5-5) :  60 * mV, min = 20 * mV, all use cases,
 --                : 600 * mV, short cable
 
  set_com_thr: process (clk)
   begin
   if rising_edge(clk) then
     if reset = '1' then
      com_adc_thr <=  60 * mV; --
     elsif rx_ena = '1' then 
        case com_thr_adj is
         when B"00"   => com_adc_thr <=     20 * mV; -- set to 100 when 3.5km filter box or 3km quad cable is being used, else use 100
         when B"01"   => com_adc_thr <=     40 * mV; -- 
         when B"10"   => com_adc_thr <=     60 * mV; -- 
         when B"11"   => com_adc_thr <=    600 * mV; -- for short cable <50m only
         when others  => com_adc_thr <=     60 * mV; --                
        end case;
      end if; -- reset = '1' 
     end if; -- rising_edge(clk)   
   end process set_com_thr; 


  com_adc_decoder: process (clk)
    begin
     if (rising_edge(clk)) then
     com_adc_clock <= not com_adc_clock; -- IMPORTANT, leave it running continously !!!
      if (rx_ena ='0') then
       com_adc_pipe    <= (others => 0);
       com_adc_sum     <= (others => 0);
      else   
       if com_adc_clock = '1' then       -- get data after LH edge
         com_adc_pipe(0) <= conv_integer(COM_ADC_D);
         com_adc_pipe(1) <= com_adc_pipe(0);
         com_adc_pipe(2) <= com_adc_pipe(1);
         com_adc_pipe(3) <= com_adc_pipe(2);
         com_adc_sum(0)  <= com_adc_pipe(0) + com_adc_pipe(1) + com_adc_pipe(2) + com_adc_pipe(3);
         com_adc_sum(1)  <= com_adc_sum(0);
         com_adc_sum(2)  <= com_adc_sum(1);
         com_adc_sum(3)  <= com_adc_sum(2);   
         com_adc_sum(4)  <= com_adc_sum(3);   
         com_adc_sum(5)  <= com_adc_sum(4);
         com_adc_sum(6)  <= com_adc_sum(5);
         com_adc_sum(7)  <= com_adc_sum(6);   
      
         com_adc_diff    <= com_adc_sum(7)- com_adc_sum(0);
         
         if com_adc_diff > com_adc_thr  then
          hl_edge <= '1';
         else
          hl_edge <= '0';
         end if; 
             
       end if; -- com_adc_clock = '1'     
      end if; -- (rx_ena ='0') 
     end if; --  (rising_edge(clk))
   end process com_adc_decoder; 

   pulse_keeper: process (clk)
    begin
     if (rising_edge(clk)) then
      if (rx_ena ='0') then
       com_adc_dec_sdout_nd <= '0';
       pulse_length_ct      <=  0;
      else 
       if com_adc_dec_sdout_nd = '0' and hl_edge = '1' then --hl_pipe = B"11" then
          com_adc_dec_sdout_nd <= '1';
          pulse_length_ct      <=  0;
       elsif  pulse_length_ct < MAX_PULSE_LENGTH -1 then
          pulse_length_ct      <= pulse_length_ct  + 1;
       elsif com_adc_dec_sdout_nd = '1' and hl_edge = '1' then --hl_pipe = B"11" then
          pulse_length_ct      <=  0; 
       else
          com_adc_dec_sdout_nd <= '0';
       end if; -- com_adc_dec_sdout_nd = '0' and hl_pipe = B"11"
      end if; --(reset ='1')
     end if; --(rising_edge(clk))
    end process pulse_keeper; 

   com_adc_dec_sdout <=  com_adc_dec_sdout_nd; 
   hl_edge_test      <= hl_edge;   
       
  COM_ADC_CLK_inst : OBUFDS
   generic map (
     IOSTANDARD => "DEFAULT")
   port map (
      O  => COM_ADC_CLK_P,   -- Diff_p output (connect directly to top-level port)
      OB => COM_ADC_CLK_N,   -- Diff_n output (connect directly to top-level port)
      I  => com_adc_clock    -- Buffer input 
   );


   COM_ADC_CSBn    <= '1';   -- serial interface disabled
   COM_ADC_SCLK    <= '0';   -- offset binary encoding selected
   COM_ADC_SDIO    <= '0';   -- normal operation

 end architecture com_adc_dec_arch;

