-------------------------------------------------------
-- Design Name : com_dac_ramp_enc 
-- File Name   : com_dac_ramp_enc.vhd
-- Function    : com_dac input signal gen. from serial data in
-- Coder       : K.-H. Sulanke, DESY
-- Date        : 2020-01-23
-- Revision    : 06
-------------------------------------------------------
-- 2020-01-23, Rev. 06, encoding bipolar pulses with ramped edges


library ieee;
    use ieee.std_logic_1164.all;
    use ieee.std_logic_unsigned.all;
    use ieee.numeric_bit.all;
    use ieee.std_logic_arith.all;
Library UNISIM;
    use UNISIM.vcomponents.all;
    
entity com_dac_ramp_enc is
   port(
        reset           : in  std_logic;
        clk             : in  std_logic;
        com_clkx2       : in  std_logic;
        baudrate_adj    : in  std_logic_vector(1 downto 0);
        sdin            : in  std_logic; -- serial data input
        com_dac_quiet   : in  std_logic; -- get zero baseline value, when comm. is inactive
        com_dac_in      : out std_logic_vector (11 downto 0); 
        com_dac_clock   : out std_logic
       );
end entity;

architecture com_dac_ramp_enc_arch of com_dac_ramp_enc is
 
  constant COM_DAC_MAX    : std_logic_vector  :=  X"fff"; -- about 1.5V at transformer 
  
  signal com_dac_din      : natural  range 0 to 4095;
  signal com_dac_clock_nd : std_logic;

  signal pulse_length      : integer range 0 to 127;
  signal half_pulse_length : integer range 0 to 63;
  
  signal ct                : natural range 0 to 29;
 
 begin
 
   get_pulse_length: process (clk) -- amount of clocks per bit
   begin
    if rising_edge(clk) then
     if reset = '1' then
      pulse_length <= 60; half_pulse_length <= 30;
     else 
       if ct = 0 then            -- for synchronized changing of the baudrate    
        case baudrate_adj is
         when B"00"   => pulse_length <=  60 ; half_pulse_length <=   30 ; --  1000_000 baud @ 60MHz clk 
         when B"01"   => pulse_length <=  40 ; half_pulse_length <=   20 ; --  1500_000 baud @ 60MHz clk 
         when B"10"   => pulse_length <=  30 ; half_pulse_length <=   15 ; --  2000_000 baud @ 60MHz clk (used now)
         when B"11"   => pulse_length <=  20 ; half_pulse_length <=   10 ; --  3000_000 baud @ 60MHz clk 
 
         when others => pulse_length <= 60; half_pulse_length <=    30 ;    
        end case;
       end if; -- ct = 0
      end if; -- reset = '1' 
     end if; -- rising_edge(clk)   
   end process get_pulse_length; 

--  counter: process (clk)
--    begin
--     if(rising_edge(clk)) then
--      if (com_dac_quiet = '1') then
--         ct <= 0;
--      else
--        if ct < pulse_length-1 then
--         ct <= ct + 1;
--        else
--         ct <= 0;  
--        end if;
--      end if; -- (com_dac_quiet = '1') 
--     end if; --rising_edge(clk) 
--    end process counter;      

 com_dac_ctrl: process (com_clkx2)
	begin
	 if (rising_edge(com_clkx2)) then
      if (com_dac_quiet = '1') then
       com_dac_din   <= 2048;--X"800"; -- zero level
       com_dac_clock_nd  <= '0';
       ct                <= 0;
      else
       com_dac_clock_nd <= not com_dac_clock_nd;
       if (sdin = '1') then
        if com_dac_clock_nd = '1' then
           if ct < 29 then
             ct <= ct + 1;
            else
             ct <= 0;
            end if;  -- if ct < 29
            if    ct < 5 then 
             com_dac_din <= com_dac_din + 409;--X"199";
            elsif ct >= 5 and ct < 10 then
             com_dac_din <= 4093;--X"FFD";
            elsif ct >= 10 and ct < 20 then
             com_dac_din <= com_dac_din - 409;-- X"199";
            elsif ct >= 20 and ct < 25 then
             com_dac_din <= 3;--X"003";
            elsif ct >= 25 and ct < 30 then
             com_dac_din <= com_dac_din + 409;--X"199";     
            end if; -- if ct < 5 
           end if; 
         else
          com_dac_din <= 2048;--X"800";
          ct          <= 0;
       end if; -- (sdin = '1')  
      end if; --  (com_dac_quiet = '1')  
     end if;  -- (rising_edge(com_clkx2)) 
    end process com_dac_ctrl; 

  com_dac_clock <= not com_dac_clock_nd;
  com_dac_in    <= conv_std_logic_vector(com_dac_din, com_dac_in'length);

 end architecture com_dac_ramp_enc_arch;
