-------------------------------------------------------
-- Design Name : half_duplex_ctrl 
-- File Name   : half_duplex_ctrl.vhd
-- Function    : controlling rx / tx arbitration
-- Coder       : K.-H. Sulanke, DESY
-- Date        : 2019-12-21
-- Revision    : 02
-------------------------------------------------------
-- 2020-01-21, full flow control implemented


library ieee;
    use ieee.std_logic_1164.all;
    use ieee.std_logic_unsigned.all;

entity half_duplex_ctrl is
   port(
        com_reset          : in  std_logic;   -- communication com_reset
        com_clk            : in  std_logic;   -- communication clock
        tx_done            : in  std_logic;   --
        rx_done            : in  std_logic;   -- last reception finished 
        ena_tx             : out std_logic;   -- initiates sending of data or token
        ena_rx             : out std_logic;   -- initiates receiving of data or token
        rx_quiet           : in  std_logic;    -- rx, no signal on the line
        rx_stf             : in  std_logic   -- start of frame received, target buffer avavilable
        );
end entity;

architecture half_duplex_ctrl_arch of half_duplex_ctrl is

  constant MAX_CABLE_DELAY  : natural := 40; -- 36us is  ~ 2x3000m cable delay
  constant FRAME_LENGTH     : natural := 15; -- 3 x 10bit @ 2MBd
  constant MAX_DELAY        : natural := 63; -- us counter wrap around
  constant RX_DELAY         : natural :=  2; -- us wait,  after TX, before RX,
  constant POWER_UP_DELAY   : natural := 50;--1023; -- time before first ena_tx after power on
  
   
  signal timer_reset   : std_logic;              -- 
  signal clk_ct        : natural range 0 to 59;  -- 60 clock 60MHz ticks, to get a us tick
  signal us            : std_logic;               -- one clock length tick
  signal us_ct         : natural range 0 to 63 ; --(MAX_CABLE_DELAY + FRAME_LENGTH)

type state_type is (POWER_UP, TX, RX_DEL,  RX_WAIT, RX);
	signal state: state_type := POWER_UP;
 
 begin
 
   get_us_ticks: process (com_clk)
    begin
     if (rising_edge(com_clk)) then	           
       if (timer_reset = '1') then
        clk_ct <= 0;
        us     <= '0';
       else
        if clk_ct < 59 then
         clk_ct <= clk_ct + 1;
         us     <= '0';
        else
         clk_ct <= 0;
         us     <= '1';
        end if; -- clk_ct < 59
       end if; -- (com_reset = '1')
     end if; -- (rising_edge(com_clk))
    end process  get_us_ticks;
          
  timer: process (com_clk)
    begin
     if (rising_edge(com_clk)) then	           
       if (timer_reset = '1') then
        us_ct    <= 0;
       elsif us = '1' then
        if us_ct < MAX_DELAY then
         us_ct <= us_ct + 1;
        else
         us_ct <= 0;
        end if;
       end if; -- (timer_reset = '1')
     end if; -- (rising_edge(com_clk))
    end process timer;
                              
   arb_sm: process (com_clk)
	   begin
	  if (rising_edge(com_clk)) then	           
       if (com_reset = '1') then
       
            state           <= POWER_UP;
            timer_reset     <= '1'; 
            ena_tx          <= '0';
            ena_rx          <= '0';
         else
			sm: case (state) is

       when POWER_UP =>  
            ena_tx          <= '0';
            ena_rx          <= '0';
            timer_reset     <= '0';
            if us_ct = POWER_UP_DELAY then
              timer_reset   <= '1';      
  			  state         <= TX;
            end if;
			              
         when TX =>  
            ena_tx          <= '1';
            ena_rx          <= '0';
            if tx_done = '1' then
              timer_reset   <= '0';       
  			  state         <= RX_DEL;
            end if;
            
         when RX_DEL =>  
            ena_tx          <= '0';
            ena_rx          <= '0';
            if us_ct = RX_DELAY then       
  			 state         <= RX_WAIT;
  			 timer_reset   <= '1'; 
            end if;         
            
         when RX_WAIT =>
            timer_reset     <= '0';  
            --ena_tx          <= '0';
            ena_rx          <= '1';
            if us_ct = (MAX_CABLE_DELAY + FRAME_LENGTH) then
             state         <= TX;
      		 timer_reset   <= '1';
      		elsif rx_stf = '1' then 
      		 state         <= RX;
      		 timer_reset   <= '1';
            end if;
  
         when RX =>
            timer_reset     <= '0';  
            if (rx_done = '1') or (rx_quiet = '1') then       
  			  state         <= TX;
   			  timer_reset   <= '1';
            end if;
        end case sm;
        
   end if; -- if (com_reset = '1')
  end if ;--if (rising_edge(com_clk))
 end process arb_sm;
 
end architecture half_duplex_ctrl_arch;
