-------------------------------------------------------
-- Design Name : rx_ctrl_8b10b 
-- File Name   : rx_ctrl_8b10b.vhd
-- Function    : controlling the 8b10b reception
-- Coder       : K.-H. Sulanke, DESY
-- Revision    : 019
-- Date        : 2020-02-10
-------------------------------------------------------
-- 2019-12-16, rev. 016 , half duplex arbitration implemented, use together with mfh_icm_top_016
-- 2020-02-07, rev. 018, full flow control implemented
-- 2020-02-10, rev. 019, rx_fifo_almost_full removed

library ieee;
    use ieee.std_logic_1164.all;

entity rx_ctrl_8b10b is
   port(
        com_reset              : in  std_logic;   -- communication com_reset
        com_clk                : in  std_logic;   -- communication clock
        rx_quiet               : in  std_logic;   -- no signal, line is quiet
        ena_rx                 : in  std_logic;   --
        rx_done                : out std_logic;   -- handshake signal with ena_rx
        dec_8b10b_out          : in  std_logic_vector (7 downto 0); --
        dec_8b10b_valid        : in  std_logic;   --
        dec_8b10b_ko           : in  std_logic;   -- decoder output is comma
        STF_COMMA_NO_RXBUF     : in  std_logic_vector (7 downto 0); -- start of frame comma
        STF_COMMA_W_RXBUF      : in  std_logic_vector (7 downto 0); -- start of frame comma
        EOF_COMMA              : in  std_logic_vector (7 downto 0);  -- end of frame comma        
        rx_fifo_wr_en          : out std_logic;   -- pulse of one clock length
        target_buf_avail       : out std_logic   -- target has buffer >= MAX_PACKET_SIZE
       );
end entity;

architecture rx_ctrl_8b10b_arch of rx_ctrl_8b10b is

 type state_type is (RX_IDLE, RX_STF, RX_DATA, RX_FWR, RX_WAIT, RX_EOF);
	signal state: state_type := RX_IDLE;
  
  signal stf_no_rxbuf        : std_logic_vector (7 downto 0);
  signal stf_w_rxbuf         : std_logic_vector (7 downto 0);  
  signal eof                 : std_logic_vector (7 downto 0); 
  signal target_buf_avail_nd : std_logic;
 begin

    stf_no_rxbuf  <= STF_COMMA_NO_RXBUF ;
    stf_w_rxbuf   <= STF_COMMA_W_RXBUF ;   
    eof           <= EOF_COMMA ;


   rx_sm: process (com_clk)
	   begin
	  sync: if (rising_edge(com_clk)) then
         
     reset_if: if (ena_rx = '0') then
            state         <= RX_IDLE;
            rx_done       <= '0';
          else
	--		sm_ena : if (dec_8b10b_valid ='1') then
    
			sm: case (state) is
         when RX_IDLE =>
            rx_done       <= '0';
            rx_fifo_wr_en <= '0';
			      if    (dec_8b10b_ko = '1') and (dec_8b10b_valid ='1') and 
			            ((dec_8b10b_out = stf_no_rxbuf) or (dec_8b10b_out = stf_w_rxbuf)) then
			        state          <= RX_STF;
			        if    (dec_8b10b_out = stf_no_rxbuf) then target_buf_avail_nd <='0';
			        elsif (dec_8b10b_out = stf_w_rxbuf)  then target_buf_avail_nd <='1';
			        end if;
			      elsif (dec_8b10b_ko = '1') and (dec_8b10b_valid ='1') and (dec_8b10b_out = eof) then
			        state          <= RX_EOF;  
            end if;  
 
         when RX_STF =>
            if     (dec_8b10b_ko = '1') and (dec_8b10b_valid ='1') and (dec_8b10b_out = eof) then
			       state         <= RX_EOF;
            elsif  (dec_8b10b_ko = '0') and (dec_8b10b_valid ='1') then
             rx_fifo_wr_en <= '1';
			       state         <= RX_FWR;
            end if;
            
         when RX_FWR =>
             rx_fifo_wr_en <= '0';
		     state         <= RX_WAIT;--RX_DATA;

         when RX_WAIT => -- i guess, this state is not really needed
             state         <= RX_DATA;			       

         when RX_DATA =>
            if     (dec_8b10b_ko = '1') and (dec_8b10b_valid ='1')  and (dec_8b10b_out = eof) then
			       state         <= RX_EOF;
            elsif  (dec_8b10b_ko = '0') and (dec_8b10b_valid ='1') then
                   rx_fifo_wr_en <= '1';
			       state         <= RX_FWR;             
            end if;              

         when RX_EOF =>
            if  rx_quiet = '1' then
               rx_done       <= '1';
            end if;
  
        end case sm;
--    end if sm_ena;
   end if reset_if;
  end if sync;
 end process rx_sm;

target_buf_avail <= target_buf_avail_nd;
 
end architecture rx_ctrl_8b10b_arch;
