-------------------------------------------------------
-- Design Name : rx_tx_led_ctrl 
-- File Name   : rx_tx_led_ctrl.vhd
-- Function    : controlling rx / tx arbitration
-- Coder       : K.-H. Sulanke, DESY
-- Date        : 2020-01-09
-- Revision    : 01
-------------------------------------------------------
-- 2020-01-09, rev.01, 

library ieee;
    use ieee.std_logic_1164.all;
    use ieee.std_logic_unsigned.all;

entity rx_tx_led_ctrl is
   port(
        com_reset          : in  std_logic;   -- communication reset
        com_clk            : in  std_logic;   -- communication clock
        tx_fifo_rd_en      : in  std_logic;   -- last sent finished 
        rx_fifo_wr_en      : in  std_logic;   -- last reception finished 
        nTX_LED_EXT        : out std_logic;   -- chassis, monitoring LEDs
        nRX_LED_EXT        : out std_logic    -- 
       );
end entity;

architecture rx_tx_led_ctrl_arch of rx_tx_led_ctrl is

  constant CLK_DIV_1000 : natural := 60_000; -- 60 MHZ / 1000
  constant ON_TIME      : natural := 500; -- 500ms
 
  signal clk_ct        : natural range 0 to CLK_DIV_1000;  -- 60MHz ticks, to get a ms tick
  signal ms            : std_logic;                        -- one clock length tick
 
  signal rx_led_ct     : natural range 0 to ON_TIME ; -- 
  signal tx_led_ct     : natural range 0 to ON_TIME ; -- 
  signal tx_led_ext    : std_logic;   -- 
  signal rx_led_ext    : std_logic;    -- 
  
 begin
 
   get_ms_ticks: process (com_clk)
    begin
     if (rising_edge(com_clk)) then	           
       if (com_reset = '1') then
        clk_ct <= 0;
        ms     <= '0';
       else
        if clk_ct < CLK_DIV_1000 then
         clk_ct   <= clk_ct + 1;
         ms       <= '0';
        else
         clk_ct <= 0;
         ms     <= '1';
        end if; -- clk_ct < CLK_DIV_1000
       end if; -- (com_reset = '1')
     end if; -- (rising_edge(com_clk))
    end process  get_ms_ticks;
          
  led_ctrl: process (com_clk)
    begin
     if (rising_edge(com_clk)) then	           
       if (com_reset = '1') then
        tx_led_ext <= '0';
        rx_led_ext <= '0';
       else
       
        if tx_fifo_rd_en = '1' then
         tx_led_ext <= '1';
         tx_led_ct  <= ON_TIME;
        elsif ms = '1' then
         if tx_led_ct /= 0 then        
          tx_led_ct  <= tx_led_ct - 1;
         else
          tx_led_ext <= '0';
         end if; --ms = '1' then
        end if;  -- tx_fifo_rd_en = '1'
    
       if rx_fifo_wr_en = '1' then
         rx_led_ext <= '1';
         rx_led_ct  <= ON_TIME;
        elsif ms = '1' then
         if rx_led_ct /= 0 then        
          rx_led_ct  <= rx_led_ct - 1;
         else
          rx_led_ext <= '0';
         end if; --ms = '1' then
        end if;  -- rx_fifo_wr_en = '1'   
    
       end if; -- (com_reset = '1')
     end if; -- (rising_edge(com_clk))
    end process led_ctrl;
                              
  nTX_LED_EXT  <= not tx_led_ext;   -- chassis, monitoring LEDs
  nRX_LED_EXT  <= not rx_led_ext;    -- 
  
end architecture rx_tx_led_ctrl_arch;
